package com.exner.tools.meditationtimer

import android.os.Build
import android.os.Bundle
import android.os.Vibrator
import android.os.VibratorManager
import android.view.WindowManager
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.activity.viewModels
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.material3.windowsizeclass.ExperimentalMaterial3WindowSizeClassApi
import androidx.compose.material3.windowsizeclass.calculateWindowSizeClass
import androidx.compose.runtime.collectAsState
import com.exner.tools.meditationtimer.audio.SoundPoolHolder
import com.exner.tools.meditationtimer.audio.VibratorHolder
import com.exner.tools.meditationtimer.data.preferences.MeditationTimerUserPreferencesManager
import com.exner.tools.meditationtimer.ui.MainViewModel
import com.exner.tools.meditationtimer.ui.MeditationTimerGlobalScaffold
import com.exner.tools.meditationtimer.ui.theme.MeditationTimerTheme
import com.exner.tools.meditationtimer.ui.theme.Theme
import dagger.hilt.android.AndroidEntryPoint
import javax.inject.Inject

@AndroidEntryPoint
class MainActivity : ComponentActivity() {

    @Inject
    lateinit var userPreferencesManager: MeditationTimerUserPreferencesManager

    private val viewModel: MainViewModel by viewModels()

//    private val dataClient by lazy { Wearable.getDataClient(this) }
//    private val capabilityClient by lazy { Wearable.getCapabilityClient(this) }

    @OptIn(ExperimentalMaterial3WindowSizeClassApi::class)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // add FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS flag to the window
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        enableEdgeToEdge()

        setContent {
            // night mode has two possible triggers:
            // - device may be in night mode
            // - force night mode setting may be on
            val userTheme = viewModel.userSelectedTheme.collectAsState()

            // window size class
            val windowSizeClass = calculateWindowSizeClass(this)

            MeditationTimerTheme(
                darkTheme = userTheme.value == Theme.Dark || (userTheme.value == Theme.Auto && isSystemInDarkTheme())
            ) {
                MeditationTimerGlobalScaffold(windowSizeClass)
            }
        }

        // experiment: vibrate
        val vibrator = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
            val vibratorManager =
                getSystemService(VIBRATOR_MANAGER_SERVICE) as VibratorManager
            vibratorManager.defaultVibrator
        } else {
            @Suppress("DEPRECATION")
            getSystemService(VIBRATOR_SERVICE) as Vibrator
        }
        VibratorHolder.initialise(vibrator)

//        // open up the nearby wearables!
//        startWearableActivity()
    }

    override fun onResume() {
        super.onResume()

//        // attach listeners for data transfer
//        dataClient.addListener(viewModel.getWearDataLayer())
//        capabilityClient.addListener(
//            viewModel.getWearDataLayer(),
//            "wear:".toUri(),
//            CapabilityClient.FILTER_REACHABLE
//        )

        // load all sounds
        SoundPoolHolder.loadSounds(this)
    }

    override fun onPause() {
        super.onPause()

        // release the kraken
        SoundPoolHolder.release()

//        // remove listeners
//        dataClient.removeListener(viewModel.getWearDataLayer())
//        capabilityClient.removeListener(viewModel.getWearDataLayer())
    }

//    // This method starts the Wearable app on the connected Wear device.
//    // Alternative to this implementation, Horologist offers a DataHelper API which allows to
//    // start the main activity or a different activity of your choice from the Wearable app
//    // see https://google.github.io/horologist/datalayer-helpers-guide/#launching-a-specific-activity-on-the-other-device
//    // for details
//    private fun startWearableActivity() {
//        lifecycleScope.launch {
//            try {
//                val nodes = capabilityClient
//                    .getCapability("wear", CapabilityClient.FILTER_REACHABLE)
//                    .await()
//                    .nodes
//
//                // Send a message to all nodes in parallel
//                // If you need an acknowledge for the start activity use case, you can alternatively use
//                // [MessageClient.sendRequest](https://developers.google.com/android/reference/com/google/android/gms/wearable/MessageClient#sendRequest(java.lang.String,%20java.lang.String,%20byte[]))
//                // See an implementation in Horologist DataHelper https://github.com/google/horologist/blob/release-0.5.x/datalayer/core/src/main/java/com/google/android/horologist/data/apphelper/DataLayerAppHelper.kt#L210
//                nodes.map { node ->
//                    async {
//                        //
//                    }
//                }.awaitAll()
//
//                Log.d("MainActivity", "Starting activity requests sent successfully")
//            } catch (cancellationException: CancellationException) {
//                throw cancellationException
//            } catch (exception: Exception) {
//                Log.d("MainActivity", "Starting activity failed: $exception")
//            }
//        }
//    }
}
